#include <vector>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <string>
#include <random>

using namespace std;

// Define RJMCMC object which specifies the settings for the RJMCMC algorithm

struct RJMCMC
{

  // Number of iterations, burn-in period and frequency of storing samples
  int iterations, burn_in, thinning;

  // Proposal probabiltiies for the specified moves
  vector<double> prob = vector<double> (7, 0.0);

  // Initial number of clusters
  int clusters;

  // Length of subperiod to estimate the matrix V
  int cut_off;
  
};


// Object to store cluster-specific information

class cluster
{
  
 public:

  // Index of cluster center
  int center;

  // Indices of sites allocated to cluster
  vector<int> sites;
  
  // Parameter values for the cluster
  double sigma, xi, gamma, eps;

  // MLE estimate
  vector<double> MLE;

  // Curvature adjustment
  vector<double> B = vector<double> ( {1.0, 0.0, 0.0, 1.0} );

};


// Object to store threshold exceedances, neighbourhood structure and chi measures

class input
{

 public:

  // Number of sites
  int K;

  // Threshold exceedances
  vector<vector<double> > peaks;

  // Highest observed threshold exceedance per site
  vector<double> max_peaks;
  
  // Threshold exceedances separated into periods
  vector<vector<vector<double> > > peaks_sep;

  // Neighbourhood structure of sites
  vector<vector<int> > neighbours;

  // Pairwise distances between sites 
  vector<vector<double> > dist;

  // Number of extreme events P_{k,k'} and Q_{k,k'}
  vector<vector<int> > P, Q;
  
  // GPD log-likelihood per site
  vector<double> llike;

};


// Object to store cluster-independent parameters

class hyperparam
{

 public:

  // Parameters for chi of municipalities allocated to different clusters
  double gamma0 = 3.0;
  double beta  = 2.0;

  // Poisson intensity of the number of clusters
  double kappa;

  // Hyperparamters for the prior/proposals of alpha, sigma, xi and beta
  double mu_sigma    = 1.0;
  double theta_sigma = 0.1;
  double mu_xi       = 0.0;
  double theta_xi    = 0.1;
  double theta_eps   = 0.01;

};


// Object to store sample output

class output
{
  
public:

  // Number of sampled clusters
  vector<int> num_clusters;
  
  // Vectors to store sampled parameters for the municipalities
  vector<vector<double> > sigma;
  vector<vector<double> > xi;
  vector<vector<double> > eps;

  // Vectors to store sampled hyperparameters
  vector<double> gamma0;
  vector<double> beta;
  vector<double> kappa;
  vector<double> mu_sigma;
  vector<double> mu_xi;
  vector<double> mu_eps;
  vector<double> theta_sigma;
  vector<double> theta_xi;
  vector<double> theta_eps;

  // Matrix to store whether municipalities are allocated to the same cluster
  vector<vector<int> > similarity;
  
};
