#include <iostream>
#include "densities.hpp"

using namespace std;


/* Derive matrix for curvature adjustment for a cluster */

void derive_B( cluster &Cluster, const input &data );


/* Compute the maximum-likelihood estimators for GPD distributed random variables */

vector<double> derive_MLE( const vector<double> &pot, const double &max_obs );


/* Derive initial estimates for the shape and scale parameter
   based on the empirical mean and variance*/

vector<double> M_Estimator( const vector<double> &pot );


/* Negative Log-likelihood function for iid threshold exceedances */

double neg_log_like_GPD(const vector<double> &obs, const double &max_obs,
			const double &sigma, const double &xi);


/* Gradient of the negative Log-likelihood function
   with respect to the scale parameter sigma */

double grad_neg_llike_GPD_sigma(const vector<double> &pot,
				const double &sigma, const double &xi);


/* Gradient of the negative Log-likelihood function
   with respect to the shape parameter xi */ 

double grad_neg_llike_GPD_xi(const vector<double> &pot,
			     const double &sigma, const double &xi);


/* Gradient of log-likelihood for iid threshold exceedances */

vector<double> llike_grad_GPD(const vector<double> &pot, 
			      const double &sigma, const double &xi);


/* First diagonal entry of the observed information matrix 
   of a GPD with parameters sigma and xi */ 

double derive_H11(const vector<double> &pot, const vector<double> &MLE);


/* Off- diagonal entry of the observed information matrix 
   for a GPD with parameters sigma and xi */ 

double derive_H12(const vector<double> &pot, const vector<double> &MLE);


/* Second diagonal entry of the observed information matrix
   for a GPD with parameters sigma and xi */ 

double derive_H22(const vector<double> &pot, const vector<double> &MLE);
