#include <vector>
#include <cstdlib>
#include <random>
#include <iostream>
#include <fstream>
#include <algorithm>
#include "analysis.hpp"

using namespace std;

int main( int argc, char* argv[] )
{

  // Read settings for the RJMCMC algorithm from "Setup.txt"
  RJMCMC Setup;
  read_Setup( Setup, argc, argv );

  /* Read peaks-over thresholds and the cardinality of P_{k,k'} and Q_{k,k'} */
  input data;
  read_data( data, Setup, argv );

  // Sample initial cluster configuration
  vector<cluster> Clusters( Setup.clusters, cluster() );
  hyperparam param;
  vector<int> no_centers, labels;
  mt19937 RNG(200000);
  init_clusters( Clusters, no_centers, labels, data, RNG, param );

  // Initialize output
  output results;
  initialize_results(data, results);
  
  // Initialize distribution to sample move
  discrete_distribution<int> sample_move ( Setup.prob.begin(), Setup.prob.end() );
  vector<int> counts (3, 0);
  
  // Run reversible jump MCMC algorithm
  for(int i=0; i<Setup.iterations; ++i){
    
    // Sample proposed move
    int move = sample_move(RNG);
    
    // Update cluster configuration according to proposed move
    if(move == 0)
      Birth( Clusters, data, labels, no_centers, param, Setup, RNG, counts[0] );
    else if(move == 1)
      Death( Clusters, data, labels, no_centers, param, Setup, RNG, counts[1] );
    else if(move == 2)
      Shift( Clusters, data, labels, no_centers, param, RNG, counts[2] );
    else if(move == 3)
      Sigma( Clusters, data, param, RNG );
    else if(move == 4)
      Xi( Clusters, data, param, RNG );
    else if(move == 5)
      Chi( Clusters, data, labels, param, RNG );
    else
      Hyper( Clusters, param, RNG );
    
    if( i >= Setup.burn_in ){

      // Update frequencies with which sites are in the same cluster
      update_similarity( Clusters, labels, results );

      // Store sampled parameters
      if( i % Setup.thinning == 0 ){
	cout << i << endl;
	store_samples( Clusters, param, labels, results );
      }

    }
    
  }

  cout<<"Birth was accepted with rate " <<
    counts[0] / (Setup.iterations * Setup.prob[0]) << endl;
  cout<<"Death was accepted with rate " <<
    counts[1] / (Setup.iterations * Setup.prob[1]) << endl;
  cout<<"Shift was accepted with rate " <<
    counts[2] / (Setup.iterations * Setup.prob[2]) << endl;
		 
  write_to_file(results, data, Setup, argv);
  
  cout << "Algorithm finished successfully" << endl;
  
}
