#include "input.hpp"


/* Birth move - Proposal to add new cluster center */

void Birth( vector<cluster> &Clusters,input &data, vector<int> &labels,
	    vector<int> &no_centers, hyperparam &param, const RJMCMC &Setup,
	    mt19937 &RNG, int &count );


/* Derive cluster labels when adding cluster at position Z_prop */

void labels_Birth(vector<cluster> &Clusters_new, const int &Z_prop,
		  vector<int> &labels_new, vector<int> &members,
		  const input &data);


/* Sample GPD parameters for proposed cluster and derive 
   prior/proposal ratio */

void Sample_Param_GPD(vector<cluster> &Clusters_new, const int &Z_new,
		      const vector<int> &members, const hyperparam &param,
		      double &accept_prob, mt19937 &RNG);


/* Sample parameters for chi for the proposed cluster and derive 
   prior/proposal ratio */

void Sample_Param_Chi(vector<cluster> &Clusters_new, const int &Z_new, 
		      const vector<cluster> &Clusters, const vector<int> &members,
		      hyperparam &param_new, double &accept_prob, mt19937 &RNG);

 
/* Derive log-likelihood ratio of GPD marginals for Birth move */

void GPD_Birth(vector<cluster> &Clusters_new, const int &Z_prop,
	       const input &data, vector<int> &members,
	       vector<double> &llike_new, double &accept_prob);


/* Death move - Proposal to remove one of the existing clusters */

void Death( vector<cluster> &Clusters, input &data,
	    vector<int> &labels, vector<int> &no_centers,
	    const hyperparam &param, const RJMCMC &Setup,
	    mt19937 &RNG, int &count);


/* Update cluster labels after deleting cluster with label Z_del */

void labels_Death(vector<cluster> &Clusters_new, const int &Z_del,
		  const cluster &C_del, vector<int> &labels_new,
		  vector<int> &members, const input &data);


/* Derive prior/proposal ratio for Death move*/

double Ratio_Death(vector<cluster> &Clusters_new,
		   const vector<cluster> &Clusters, const cluster &C_del,
		   const vector<int> &members, const hyperparam &param);
  

/* Derive log-likelihood ratio of GPD marginals for Death move */

void GPD_Death(vector<cluster> &Clusters_new, const input &data,
	       vector<int> &members, vector<double> &llike_new,
	       double &accept_prob);


/* Shift move - Proposal to shift new cluster center to an adjacent
   site that is not cluster center */

void Shift(vector<cluster> &Clusters, input &data,
	   vector<int> &labels, vector<int> &no_centers,
	   const hyperparam &param, mt19937 &RNG,
	   int &count);


/* Remove indices of current cluster centres */

void remove_centres(vector<int> &candid, const vector<int> &no_centers);


/* Update cluster labels after shifting one cluster center */

void labels_Shift(vector<cluster> &Clusters_new,
		  const int &Z_shift, vector<int> &labels_new,
		  vector<int> &members, const input &data);


/* Derive likelihood ratio of the GPD marginals for Shift move */

void GPD_Shift(vector<cluster> &Clusters_new, const input &data,
	       vector<int> &members, vector<double> &llike_new,
	       double &accept_prob);


/* Update GPD scale parameter sigma for all clusters */

void Sigma(vector<cluster> &Clusters, input &data,
	   const hyperparam &param, mt19937 &RNG);


/* Update GPD shape parameter xi for all clusters */

void Xi(vector<cluster> &Clusters, input &data,
	       const hyperparam &param, mt19937 &RNG);


/* Update parameters for distribution of Chi */ 

void Chi(vector<cluster> &Clusters, const input &data,
	 const vector<int> &labels, hyperparam &param,
	 mt19937 &RNG);


/* Update hyper parameters */

void Hyper( const vector<cluster> &Clusters,
	    hyperparam &param, mt19937 &RNG );
