library(fExtremes)
library(RColorBrewer)
library(lattice)
library(mvtnorm)
library(ismev)
library(sp)

load("Mapping.RData")
source("Neighbourhood StructureNorway.R")
gadm <- readRDS("NOR_adm2.rds")

## Select regions of Ostfold, Akershus and eastern Buskerud - 55 sites
sites             <- c( 1:41, 79, 80, 81, 82, 84, 92:94, 100:102, 105:107 )
labels_map        <- rep( NA, 430 )
labels_map[sites] <- 1

mydata_new  <- labels_map
mydata_plot <- mydata_new[mapping]
gadm$clusters <- mydata_plot

## Extract neighbourhood structure
neighbours <- vector("list", length(sites) )
for( i in 1:length(sites) ){
  temp <- neighbours_true[[ sites[i] ]]
  for(k in temp){
    ind <- which( sites == k )
    neighbours[[i]] <- c(neighbours[[i]], ind)
  }
}


## Derive distance between sites on [0,1] scale
K    <- length(sites)
coord_municip <- matrix(0, K, 2)
for(j in 1:K){
  k <- sites[j]
  ind <- which( mapping == k )
  for(i in ind)
    coord_municip[j,] <- coord_municip[j,] + gadm@polygons[[i]]@labpt
  coord_municip[j,] <- coord_municip[j,] / length(ind)
}
coord_municip[23,] <- gadm[35,]@polygons[[1]]@Polygons[[2]]@labpt
coord_municip[37,] <- gadm[35,]@polygons[[1]]@Polygons[[1]]@labpt

dist <- matrix( 0, K, K )
for(k in 1:K){
  for(j in 1:K){
    phi1    <- coord_municip[k,2]
    phi2    <- coord_municip[j,2]
    lambda1 <- coord_municip[k,1]
    lambda2 <- coord_municip[j,1]
    temp_a <- sin( ( phi1 - phi2 ) *pi / 360 ) ^ 2 + 
      cos(phi1 * pi / 180) * cos(phi2 * pi / 180) * sin( ( lambda1 - lambda2 ) * pi / 360 ) ^ 2
    temp_c <- 2 * atan2( sqrt(temp_a), sqrt(1 - temp_a) )
    dist[k,j] <- 6371 * temp_c
  }
}
dist <- dist / max(dist)


############## Sample clusters ####################
set.seed(2019)
J <- 1
Z <- rep(0, length(sites))

## Model parameters ##
n     <- 100
psi   <- 2.0
nu    <- 0.2
gamma <- 2.0
beta  <- 10

for(i in 1:100){

  ## Store distance and neighbourhood structure
  write.table( dist, paste("Run",i,"/Distances.txt",sep=""), 
               sep=" ", row.names=F, col.names=F )
  write.table( t(neighbours[[1]]), paste("Run",i,"/Neighbours.txt",sep=""), 
               sep=" ", row.names=F, col.names=F )
  for(k in 2:K)
    write.table( t(neighbours[[k]]), paste("Run",i,"/Neighbours.txt",sep=""), 
                   sep=" ", row.names=F, col.names=F, append=T )

  ## Sample peaks over threshold  
  obs <- matrix(0, K, n)
  for(k in 1:K)
    obs[k,] <- rgpd( n, xi=nu, beta=psi )
  write.table(obs, paste("Run",i,"/PeaksOverThreshold.txt", sep="" ),
              sep=" ", col.names=F, row.names=F)

  ## Sample extremal dependence measures
  Q <- matrix( 25, K, K )
  P <- matrix(  0, K, K )
  alpha <- 0
  for(k in 1:K){
    for(j in k:K){
      alpha  <- beta / ( exp( gamma * dist[k,j] ) - 1 )
      chi    <- rbeta(1, alpha, beta) 
      P[k,j] <- P[j,k] <- rbinom(1, Q[k,j], chi)
    }
  }
  write.table(P, paste("Run",i,"/P.txt", sep="" ), sep=" ", col.names=F, row.names=F)
  write.table(Q, paste("Run",i,"/Q.txt", sep="" ), sep=" ", col.names=F, row.names=F)
  
}
