library(fExtremes)
library(ismev)
library(lattice)
library(kernlab)
library(mcclust.ext)
library(mcclust)
library(sp)

#### Analysis posterior samples for the number of clusters J ####

#### Analysis posterior samples for the number of clusters J ####

## Import file with the samples for J, gamma0, beta and the hyperparameters
samples <- read.table( "Number of clusters and hyperparameters.txt", header=T )

## Trace plot for number of clusters J
pdf( "SampledJSummer.pdf", width=5, height=4 )
par( mai=c(1,1,0.3,0.3) )
plot( samples$Number_of_clusters, type='l', ylab="Number of Clusters", 
      xlab="Sample", cex.lab=1.5, cex.axis=1.5 ) 
dev.off()

## Histrogram for posterior samples of J
pdf( "HistogramJSummer.pdf", width=5, height=4 )
par( mai=c(1,1,0.3,0.3) )
temp <- min(samples$Number_of_clusters) : max(samples$Number_of_clusters)
freq <- rep(0, length(temp) )
for(i in 1:length(temp))
  freq[i] <- sum(samples$Number_of_clusters == temp[i]) / length(samples$Number_of_clusters)
plot(temp, freq, type='h', xlab="Number of Clusters", ylab="Frequency",
     cex.lab=1.5, cex.axis=1.5, xlim=c(1,25))
dev.off()

#### Derive cluster estimate Z* ####
similarity <- as.matrix( read.table("Similarity.txt") )
Z_star     <- minVI(similarity)$cl

neighbours <- scan("Neighbours.txt", what="", sep="\n")
neighbours <- strsplit(neighbours, "[[:space:]]+")
neighbours <- lapply(neighbours, as.numeric)

## Plot cluster estimate Z* on hydroglocial space
locations <- read.csv(file = "../RiverFlowLocations.csv", header=T)
locations <- locations[-31,]

pdf( "ClusteringHydroSummer.pdf", width=5, height=5 )
par( mai=c(0.1,0.1,0.1,0.1) )
myPalette <- 2:( max(Z_star) + 1 )
plot( locations$HydroEast, locations$HydroNorth, col = myPalette[Z_star], 
      xlab="", ylab="", yaxt='n', xaxt='n', pch=19, cex.axis=1.5, cex=2.8 )
for(k in 1:45){
  for(j in neighbours[[k]]){
    color <- 8
    if( Z_star[k] == Z_star[j] )
      color <- 8
    lines( c(locations$HydroEast[k] , locations$HydroEast[j] ),
           c(locations$HydroNorth[k], locations$HydroNorth[j]),
           lwd = 1 + 5 * similarity[k,j], col = color)
  }
}
points( locations$HydroEast, locations$HydroNorth, 
        col = myPalette[Z_star], pch=19, cex=2.8 )
dev.off()

## Plot cluster estimates on UK map and highlight Marple Bridge and Kirkby Stephen 
pdf("ClusteringGeoSummer.pdf", width=5, height=5)
par(mai=c(0.1,0.1,0.1,0.1))
gadm <- readRDS("../gadm36_GBR_2_sp.rds")
plot(gadm, xlim=c(-3.5, -1.5), ylim=c(53.2, 55.2), cex.lab=2,  col = 8, lwd=0.001 )
points(locations$Lon, locations$Lat, pch=19, col=myPalette[Z_star], cex = 2.1)
dev.off()



#### Plot and check posterior estimates for Marple Bridge and Kirkby Stephen ####

site_index    <- c( 17,41 )
name_gauge    <- c( "Marple", "Kirkby" )
pot           <- as.matrix( read.table( "PeaksOverThreshold.txt") )
obs_unscaled  <- as.matrix( read.table( "AggregatedFlow.txt") )
samples_sigma <- as.matrix( read.table( "Sigma.txt") )
samples_xi    <- as.matrix( read.table( "Xi.txt") )    

for( k in site_index ){
  
  ## Trace plot for the scale parameter
  pdf(paste(name_gauge[which(site_index==k)],"PsiSummer.pdf", sep=""), width=5, height=4 )
  par(mai=c(1,1,0.3,0.3))
  plot( samples_sigma[k,], ylab="Parameter Value", xlab="Sample",
        cex.lab=1.5, cex.axis=1.5, type='l' )
  dev.off()
  
  ## Trace plot for the shape parameter
  pdf(paste(name_gauge[which(site_index==k)],"NuSummer.pdf", sep=""), width=5, height=4 )
  par(mai=c(1,1,0.3,0.3))
  plot( samples_xi[k,], ylab="Parameter Value", xlab="Sample", 
        cex.lab=1.5, cex.axis=1.5, type='l' )
  dev.off()
  
  ## Derive QQ plots
  pdf( paste(name_gauge[which(site_index==k)],"Summer.pdf", sep=""), width=5, height=5 )
  par( mai=c(1,1,0.3,0.3) )
  num_ext <- sum( pot[k,] > 0 )
  quant_k <- matrix( 0, num_ext, length(samples_sigma[1,]) )
  for( j in 1:length(samples_sigma[1,]) )
    quant_k[,j] <- qgpd( (1:num_ext) / (num_ext+1), 
                         beta = samples_sigma[k,j], xi = samples_xi[k,j] )
  ind <- which( pot[k,] > 0 )
  qqplot( apply(quant_k, 1, mean)+2, pot[k,ind]+2 ,ylab="Empirical values",
          xlab="Model estimates", cex.lab=1.5, cex.axis=1.5 ) 
  abline(0,1)
  lines( apply(quant_k, 1, quantile, 0.025)+2, sort(pot[k,ind])+2, col=2, lty=2 ) 
  lines( apply(quant_k, 1, quantile, 0.975)+2, sort(pot[k,ind])+2, col=2, lty=2 ) 
  dev.off()
  
  ## Check mixing of the log-likelihood for the site and compare to value for MLE
  llike <- rep(0, length(samples_sigma[k,]))
  for(j in 1:length(samples_sigma[k,]) )
    llike[j] <- sum( dgpd(pot[k,ind], beta = samples_sigma[k,j], xi = samples_xi[k,j], log=T) )
  fit_MLE <- -gpd.fit(pot[k,ind], threshold = 0, show=F)$nllh
  
  pdf( paste(name_gauge[which(site_index==k)],"LikelihoodSummer.pdf", sep=""), width=5, height=4 )
  par(mai=c(1,1,0.3,0.3))
  plot( llike, type='l', ylab="Log-likelihood", xlab="Sample", 
        cex.lab=1.5, cex.axis=1.5, ylim=c(min(llike), fit_MLE) )
  abline(h = fit_MLE, col=2)
  dev.off()    
  
  ## Derive return level estimates on transformed scale
  quant_sites_100 <- 2.0 + samples_sigma[k,] / samples_xi[k,] * 
    ( (sum(pot[k,]>0, na.rm=T) / sum( is.na(pot[k,]) == F) * 20 * 100) ^ samples_xi[k,] - 1)
  quant_sites_500 <- 2.0 + samples_sigma[k,] / samples_xi[k,] * 
    ( (sum(pot[k,]>0, na.rm=T) / sum( is.na(pot[k,]) == F) * 20 * 500) ^ samples_xi[k,] - 1)
  
  ## Derive return level estimates on original scale
  quant_sites_100 <- quant_sites_100 * sd( obs_unscaled[k,], na.rm=T ) + 
    mean( obs_unscaled[k,], na.rm=T )
  quant_sites_500 <- quant_sites_500 * sd( obs_unscaled[k,], na.rm=T ) + 
    mean( obs_unscaled[k,], na.rm=T )
  print( paste("100-year return level for",name_gauge[which(site_index==k)], sep=""))
  print( c( quantile(quant_sites_100, 0.05), 
            mean(quant_sites_100),  
            quantile(quant_sites_100, 0.95) ) )
  print( paste("500-year return level for",name_gauge[which(site_index==k)], sep=""))
  print( c( quantile(quant_sites_500, 0.05), 
            mean(quant_sites_500),  
            quantile(quant_sites_500, 0.95) ) )
  
}


#### Extract data for the FC approach for Marple  Bridge and Kirkby Stephen ####
P          <- read.table("P.txt")
Q          <- read.table("Q.txt")
hydro_dist <- read.table("Distances.txt")

## Extract data for the FC approach for Marple Bridge
ind <- which( Z_star == Z_star[17] )
write.table(pot[ind,], "Marple Bridge/PeaksOverThreshold.txt", col.names=F, row.names=F)
write.table(P[ind,ind], "Marple Bridge/P.txt", col.names=F, row.names=F)
write.table(Q[ind,ind], "Marple Bridge/Q.txt", col.names=F, row.names=F)

neighbours_k <- vector("list", length(ind) )
for(j in 1:length(ind) ){
  neighbours_k[[j]] <- neighbours[[ ind[j] ]]
  neighbours_k[[j]] <- intersect( neighbours_k[[j]], ind )
  for( i in 1:length(neighbours_k[[j]]) )
    neighbours_k[[j]][i] <- which( ind == neighbours_k[[j]][i] )
}
write.table( t(neighbours_k[[1]]), "Marple Bridge/Neighbours.txt", 
             col.names = F, row.names = F )
for(j  in 2:length(ind) )
  write.table( t(neighbours_k[[j]]), "Marple Bridge/Neighbours.txt", 
               col.names = F, row.names = F, append = T )

## Extract data for the FC approach for Kirkby Stephen
ind <- which( Z_star == Z_star[41] )
write.table(pot[ind,], "Kirkby Stephen/PeaksOverThreshold.txt", col.names=F, row.names=F)
write.table(P[ind,ind], "Kirkby Stephen/P.txt", col.names=F, row.names=F)
write.table(Q[ind,ind], "Kirkby Stephen/Q.txt", col.names=F, row.names=F)

neighbours_k <- vector("list", length(ind) )
for(j in 1:length(ind) ){
  neighbours_k[[j]] <- neighbours[[ ind[j] ]]
  neighbours_k[[j]] <- intersect( neighbours_k[[j]], ind )
  for( i in 1:length(neighbours_k[[j]]) )
    neighbours_k[[j]][i] <- which( ind == neighbours_k[[j]][i] )
}
write.table( t(neighbours_k[[1]]), "Kirkby Stephen/Neighbours.txt", 
             col.names = F, row.names = F )
for(j  in 2:length(ind) )
  write.table( t(neighbours_k[[j]]), "Kirkby Stephen/Neighbours.txt", 
               col.names = F, row.names = F, append = T )